function f = BTS (x, P, v_dim, k_interior, M)

%% Explanations:
% * P = size of the mating pool
% * x = input-vector containing population data

% Simplifing notation
nbFixedPoints = 3; % i.e. the start point and the destination
V = v_dim*(k_interior + nbFixedPoints);

% Retrieving the rank and crowding distance from the population data:
rank_index = V+M+1;
cdist_index = V+M+2;

%% Binary tournament selection
% Two random solutions are compared in order to figure out which one is
% best fit to be a parent.
% The comparison is based on: 1) lower rank is preferred, 2) if the ranks
% are equal, the solution with the higher crowding distance is preferred

% Generating random unique indicies for which individuals to compare
rand_nb = randperm(2*P)';
% testout = sort(rand_nb)

compare_i(:,1) = rand_nb(1:P);
compare_i(:,2) = rand_nb(P+1:end);

%compare_i = [randperm(P); randperm(P)]';

% Pre allocation
mating_pool = zeros(P,V+M+2);

for p = 1:P
    
    % Contender 1
    contender_1 = compare_i(p,1);
    rank_1 = x(contender_1, rank_index);
    
    % Contender 2
    contender_2 = compare_i(p,2);
    rank_2 = x(contender_2, rank_index);
    
    % If contenders have different rank
    if rank_1 ~= rank_2
        
        if rank_1 < rank_2
            winner = contender_1;
        else %if rank_1 > rank_2
            winner = contender_2;
        end
        
        mating_pool(p,:) = x(winner,:);
    
    % if contenders have the same rank    
    else 
        % Retrieving the crowding distance for the contenders
        cdist_1 = x(contender_1, cdist_index);
        cdist_2 = x(contender_2, cdist_index);
        
        if cdist_1 > cdist_2 
            winner = contender_1;
        else % cdist_1 < cdist_2
            winner = contender_2;
%         else %if equal crowding distance
%             winner = contender_1; % selecting the first one for simplicity
        end
        
        mating_pool(p,:) = x(winner,:);
    end 
    
end


f = mating_pool;
end