function f  = NDS_CDA_rev2(x, v_dim, k_interior, M)

% Simlifying notation for user readability
[N, ~] = size(x);

nbFixedPoints = 3; % i.e. the start point and the destination
V = v_dim*(k_interior + nbFixedPoints);

% Extracting objective function values
f_1 = x(:,V+1);
f_2 = x(:,V+2);
f_3 = x(:,V+3);
f_4 = x(:,V+4);


%% Explanations:
% * Domination count, n_p: The number of solutions that dominate the
% solution p. Here expressed by notation: sol(p).n
% * S_p: A set of solutions that the solutions p dominates. Here expressed 
% by notation: sol(p).S
% * This code is based on pseudo-code from the article: 
%   https://ieeexplore.ieee.org/document/996017  

%% Fast non-dominated sorting

% Pre-allocation
sol(:).n = 0;
sol(:).sp = [];
rank = zeros(N,1);
f_i = 1; 
F(f_i).f = [];

for p = 1:N
    % Domination count, initialted n_p = 0:
    sol(p).n = 0;
    % Set Sp, initially an empty set:
    sol(p).S = [ ]; 
    
    for q = 1:N
        
        % if p dominates q
        if f_1(p) < f_1(q) && f_2(p) < f_2(q) && f_3(p) < f_3(q) && f_4(p) < f_4(q) ...
                || f_1(p) == f_1(q) && f_2(p) < f_2(q) && f_3(p) < f_3(q) && f_4(p) < f_4(q)...
                || f_1(p) < f_1(q) && f_2(p) == f_2(q) && f_3(p) < f_3(q) && f_4(p) < f_4(q)...
                || f_1(p) < f_1(q) && f_2(p) < f_2(q) && f_3(p) == f_3(q) && f_4(p) < f_4(q)...
                || f_1(p) < f_1(q) && f_2(p) < f_2(q) && f_3(p) < f_3(q) && f_4(p) == f_4(q)
            
            % Add q to the set of solutions dominated by p
            sol(p).S = [sol(p).S, q];
         
        % if q dominates p
        elseif f_1(p) > f_1(q) && f_2(p) > f_2(q) && f_3(p) > f_3(q) && f_4(p) > f_4(q)...
                || f_1(p) == f_1(q) && f_2(p) > f_2(q) && f_3(p) > f_3(q) && f_4(p) > f_4(q)...
                || f_1(p) > f_1(q) && f_2(p) == f_2(q) && f_3(p) > f_3(q) && f_4(p) > f_4(q)...
                || f_1(p) > f_1(q) && f_2(p) > f_2(q) && f_3(p) == f_3(q) && f_4(p) > f_4(q)...
                || f_1(p) > f_1(q) && f_2(p) > f_2(q) && f_3(p) > f_3(q) && f_4(p) == f_4(q)
                
            % Increment the domination counter of p
            sol(p).n = sol(p).n + 1;
            
        end % if domination
        
    end % for q
    
    % Identifying the first non-dominated front
    if sol(p).n == 0
        rank(p,1) = 1;
        F(f_i).f = [F(f_i).f, p];
    end
    
    
end %for p

x(:, V + M + 1) = rank;

% Identifying subsequent non-dominated fronts 
while ~isempty(F(f_i).f)
    
    % Empty set list used to store members of the next front
    Q = [ ];
    
    % For each solution p with n_p = 0, visiting each member if its set S_p
    % and reducing its domination count by one
    for p = 1:length(F(f_i).f)
        
            for q = 1:length( sol( F(f_i).f(p) ).S )

                % Reducing domination count by one
                sol( sol( F(f_i).f(p) ).S(q) ).n = ...
                    sol( sol( F(f_i).f(p) ).S(q) ).n -1;

                % if n_p = 0 for any members, these members are put in the 
                % list Q as they belong to the next non-dominated front
                if sol( sol( F(f_i).f(p) ).S(q) ).n == 0

                    x( sol(F(f_i).f(p)).S(q), V + M + 1) = f_i + 1;
                    %rank(p,1) = f_i + 1; % PROBLEM HERE??
                    Q = [Q sol(F(f_i).f(p)).S(q)];

                end % end if n_p = 0
            end
    end
    
    % Incrementing front counter
    f_i = f_i + 1;
    F(f_i).f = Q;
    
end

% Sorting the population based on acending rank
x_r_s = sortrows(x,V + M + 1);


%% Crowding distance assignment
% Calculating the crowding distance for each solution in each non-dominated 
% set, i.e in each front

% Making a temporary population data vector
x_temp = x_r_s;


row_i = 1;
for i = 1: ( length(F) - 1 )
    
    % Initializing distance, i_dist = 0
    i_dist = 0;
    
    % Retrieving the length of each front 
    length_f = length(F(i).f);
    
    % If the length of the front is higher than 2 intermediate solutions
    % exists
    if length_f > 2
        
        % Sorting the population based on objective function values
        [~, f_1_i] = sortrows(x_temp(row_i:(row_i+length_f - 1) , V + 1));
        [~, f_2_i] = sortrows(x_temp(row_i:(row_i+length_f - 1) , V + 2));
        [~, f_3_i] = sortrows(x_temp(row_i:(row_i+length_f - 1) , V + 3));
        [~, f_4_i] = sortrows(x_temp(row_i:(row_i+length_f - 1) , V + 4));
        
        % Assigning boundary solutions infinite distance
        x_temp( f_1_i(1) + row_i - 1 , V + M + 2 ) = inf;
        x_temp( f_1_i(end) + row_i - 1 , V + M + 2 ) = inf;
        
        x_temp( f_2_i(1) + row_i - 1 , V + M + 3 ) = inf;
        x_temp( f_2_i(end) + row_i - 1 , V + M + 3 ) = inf;
        
        x_temp( f_3_i(1) + row_i - 1 , V + M + 4 ) = inf;
        x_temp( f_3_i(end) + row_i - 1 , V + M + 4 ) = inf;
        
        x_temp( f_4_i(1) + row_i - 1 , V + M + 5 ) = inf;
        x_temp( f_4_i(end) + row_i - 1 , V + M + 5 ) = inf;
        
        % Retrieving objective function extreme values for normalization
        f_1_min = x_temp(f_1_i(1) + row_i - 1 , V + 1);
        f_1_max = x_temp(f_1_i(end) + row_i - 1 , V + 1);
        
        f_2_min = x_temp(f_2_i(1) + row_i - 1 , V + 2);
        f_2_max = x_temp(f_2_i(end) + row_i - 1 , V + 2);
        
        f_3_min = x_temp(f_3_i(1) + row_i - 1 , V + 3);
        f_3_max = x_temp(f_3_i(end) + row_i - 1 , V + 3);
        
        f_4_min = x_temp(f_4_i(1) + row_i - 1 , V + 4);
        f_4_max = x_temp(f_4_i(end) + row_i - 1 , V + 4);
        
        % Assigning intermediate solutions a distance
        for j = 2:length_f - 1
            
           % If more boundary solutions exists 
           if (f_1_min - f_1_max == 0) || (f_2_min - f_2_max == 0)...
                   || (f_3_min - f_3_max == 0) 
               x_temp (f_1_i(j)+row_i-1, V+M+2) = inf;
               x_temp (f_2_i(j)+row_i-1, V+M+3) = inf;
               x_temp (f_3_i(j)+row_i-1, V+M+4) = inf;
           % Intermediate solutions - normalized 
           else
               x_temp (f_1_i(j)+row_i-1, V+M+2) =...
                   (x_temp(f_1_i(j+1) + row_i - 1, V+1)...
                   - x_temp(f_1_i(j-1) + row_i - 1, V+1))...
                   /(f_1_max - f_1_min);
               
               x_temp (f_2_i(j)+row_i-1, V+M+3) =...
                   (x_temp(f_2_i(j+1) + row_i - 1, V+2)...
                   - x_temp(f_2_i(j-1) + row_i - 1, V+2) )...
                   /(f_2_max - f_2_min);
               
               x_temp (f_3_i(j)+row_i-1, V+M+4) =...
                   (x_temp(f_3_i(j+1) + row_i - 1, V+3)...
                   - x_temp(f_3_i(j-1) + row_i - 1, V+3) )...
                   /(f_3_max - f_3_min);
               
               x_temp (f_4_i(j)+row_i-1, V+M+5) =...
                   (x_temp(f_4_i(j+1) + row_i - 1, V+4)...
                   - x_temp(f_4_i(j-1) + row_i - 1, V+4) )...
                   /(f_4_max - f_4_min);
           end
        end
    
    % Else only boundary solutions exists
    else
        % Assigning boundary solutions infinite distance
        x_temp(row_i:(row_i+length_f-1), V+M+2:V+M+5) = inf;
    end
    row_i = row_i + length_f;
end
% Calculating the overall crowding distance, i.e. the sum of the individual
% crowding distances. 
crowding_dist = zeros(N,1);
for i = 1:N
    crowding_dist(i) = x_temp(i,V+M+2) + x_temp(i,V+M+3) +...
        x_temp(i,V+M+4) + x_temp(i,V+M+5);
end
x_r_s(:, V + M + 2) = crowding_dist;

% Test
f = x_r_s;

end
