function f = OFE_multiple_obstacles_m4( vec,...
                K, V, min_dist, obs_vec, z_terrain)

%% Directory: Dir_3/3 objectives            
%% Objective function f(1): Length of path - low f(1) is good
% Count indicies:
xc = 1;
yc = 2;
zc = 3;

df_1 = zeros(1,K+2);
for i = 1:K+2

    % Length of path segment between two adjacent waypoints
    df_1(i) = sqrt( ( vec(xc+V) - vec(xc) )^2 + ...
                 ( vec(yc+V) - vec(yc))^2 + ...
                 ( vec(zc+V) - vec(zc))^2 );
             
    % Updating counting idicies
    xc = xc + V;
    yc = yc + V;
    zc = zc + V;
    
    %f_1 = f_1 + df_1(i);
end
clear xc yc zc;
f(1) = sum(df_1);
%f_path = sum(df_1);

%% - Objective function f(2): Margin of safety 
% more negative f(2) is better
% Calculating the shortest distance between every line segment and the
% obstacle.

%Number of obstalces
nb_obs = length(obs_vec)/3;
obs_mat = zeros(nb_obs, 3);

% Count indicies:
xc = 1;
yc = 2;
zc = 3;

for i = 1:nb_obs
    obs_mat(i,:) =  [obs_vec(xc), obs_vec(yc), obs_vec(zc)];
        % Updating counting idicies
    xc = xc + V;
    yc = yc + V;
    zc = zc + V;
end

dist = zeros(nb_obs,K+1);

% for each obstacle
for j = 1:nb_obs
    % Count indicies:
    xc = 1;
    yc = 2;
    zc = 3;

    for i = 1:K+1
        % Adjacent waypoints on vetor form
        wp1 = [vec(xc), vec(yc), vec(zc)];
        wp2 = [vec(xc+V), vec(yc+V), vec(zc+V)];

        % Reference: Algorithm used for calculating distance from point to line:
        % http://mathworld.wolfram.com/Point-LineDistance3-Dimensional.html
        num = norm(cross(obs_mat(j) - wp1, obs_mat(j) - wp2));
        den = df_1(i);
        dist(j,i) = num / den;

        % Updating counting idicies
        xc = xc + V;
        yc = yc + V;
        zc = zc + V;
    end

end
clear xc yc zc;

dist_vec = [ dist(1,:), dist(2,:)];

% Calculating the objective function value. 
d_min = min(dist_vec);

if (d_min < min_dist)
    % If the path is situated less than a safety distance away from the
    % obstale, f_2 will approach zero or even be positive
    f(2) = exp(d_min-min_dist);
else
    % If the path is situated more than a safety distance away from the
    % obstacle, the f_2 will take a negative value. 
    f(2) = min_dist - d_min;
end

%% Objective function f(3) - penaltizing sharp turns in paths

xc = 1;
yc = 2;

theta = zeros(1,K);

for i = 1:K
    wp1 = [vec(xc), vec(yc)];
    wp2 = [vec(xc + V), vec(yc + V)];
    wp3 = [vec(xc + 2*V), vec(yc + 2*V)];
    
    dp_1 = wp2 - wp1;
    dp_2 = wp3 - wp2;
    
    theta(i) = acos( (dot(dp_1, dp_2))/(norm(dp_1)*norm(dp_2)) );
    
    xc = xc + V;
    yc = yc + V;
end


f_3_theta = max(abs(theta));

f(3) = f_3_theta;

%% Objective function f(4) - ensuring available DVL measurements
zc = V + V;
z_wp = zeros(1,K);

for i = 1:K
    z_wp(i) = abs(vec(zc) - z_terrain) ;
    zc = zc + V;
end

f_terr = zeros(1,K);
z_limit = 10;
for i = 1:length(z_wp)
    z_dev = z_wp(i);
    if z_dev > z_limit
        f_terr(i) = exp((z_limit - z_dev));
    else % z_dev < z_limit
        f_terr(i) = z_dev - z_limit;
    end
end
f_terr_pen = max(f_terr); % Sum or max, which is better?

f(4) = f_terr_pen;












end %fcn