function f = initialize_population(N, v_dim, k_interior, M,...
                                    start_pos, final_pos,...
                                    min_range, max_range,...
                                    min_dist, obs_vec, z_terrain, pen_pos)
%% Description
% *************************************************************************
% This function generates an initial population, f, of paths  at random 
% based on the input from the file parameters.m
%
% This function calls the function OFE_multiple_obstacles for evaluation of
% the objective function values of the generated paths.
%
% For explanation of inputs, see parameters.m / Main.m
% *************************************************************************
% Programmed by: Tobias Lars Hansen, 2019.
% *************************************************************************
%%
nbFixedPoints = 3; 
V = v_dim*(k_interior + nbFixedPoints);

pop_data = zeros(2*N, V + M + 2);
for i = 1:2*N
    %% Generate random population of size N, i.e. generate N random paths
    % between start and destination 
    
    % Assigning starting point
    for j = 1:v_dim
        pop_data(i,j) = start_pos(j);
    end
    % Assigning internal points
    count = v_dim + 1;
    for j = 1:k_interior
        for k = 1:v_dim
            pop_data(i,count) = min_range(k) +...
                (max_range(k)-min_range(k))*rand(1);
            count = count + 1;
        end
    end
    clear count;
    % Assigning penultimate point
    for j = 1:v_dim
       pop_data(i, V - 2*v_dim + j) = pen_pos(j); 
    end
    
    % Assigning destination point
    for j = 1:v_dim
        %pop_data(i,V+V*K+j) = final_pos(j);
        pop_data(i,V - v_dim + j) = final_pos(j);
    end 
    %% Calculating the objective function value for each individual, i.e.
    % evaulate the performance of each path
    pop_data(i,V+1:V+M) = ...
        OFE_multiple_obstacles(pop_data(i,:), k_interior, v_dim, ...
                                min_dist, obs_vec,z_terrain);
end
f = pop_data;
end %fcn