%% Description:
% *************************************************************************
% This file contains the parameters settings for NSGA-II and the parameters
% defining the optimal path planning problem for autonomous docking
% missions.
%
% This file should be modified if desireable to solve the path planning
% problem for different scenarios.
%
% For explanation of the parameters, please read comments throughout the
% code and/or see:
% Hansen, Tobias Lars, "A NSGA-II Based Optimal Path Planner for Autonomous
% Docking of ROV Using Waypoint Guidance", 2019. 
% *************************************************************************
% Programmed by: Tobias Lars Hansen, 2019.
% *************************************************************************
%% Parameters
deg2rad = pi/180;
%% NSGA-II parameters:
N = 100; % Population size
G = 500; % Stopping criterion, number of generations
V = 3; 
M = 4; % Number of objective functions

%% Parameters for Genetic Operations
% Simulated Binary Crossover (SBX) parameters
p_c = 0.9; % Crossover probability
eta_c = 5; % Distribution index, 20 suggested by literature
% Polynomial Mutation
p_m = 0.1; % Mutation problability
eta_m = 20; % Distribution index

%% Environment parameters
% Initial position
x0 = 1;
y0 = 1;
z0 = 1;
start_pos = [x0, y0, z0];

% Destination
x_final = 100;
y_final = 100;
z_final = 190;
final_pos = [x_final, y_final, z_final];

% Final heading
heading_final = -90*deg2rad; % 0 deg is towards north

% Length of last path segment
length_last_segment = 20;

% Penultimate position
x_pen = final_pos(1) + cos(heading_final)*(-length_last_segment);
y_pen = final_pos(2) - sin(heading_final)*(-length_last_segment);
z_pen = z_final;
pen_pos = [x_pen, y_pen, z_pen];

% Variable bounds
xxmin = [x0, x_final, x_pen];
xxmax = [x0, x_final, x_pen];
yymin = [y0, y_final, y_pen];
yymax = [y0, y_final, y_pen];

x_min = min(xxmin) - 1;
x_max = max(xxmax) + 1;

y_min = min(yymin) - 1;
y_max = max(yymax) + 1;

z_min = 30;
z_max = 70;

% Range vectors
max_range = [x_max, y_max, z_max];
min_range = [x_min, y_min, z_min];

% f(4)
z_terrain = z_max - 10; 

K = 4; % Number of interior waypoints

% Obstacle position(s)
x_obs = 50;
y_obs = 50;
z_obs = 60;
% obs_vec = [x_obs, y_obs, z_obs];

x_obs_2 = 30;
y_obs_2 = 30;
z_obs_2 = 40;
obs_vec = [x_obs, y_obs, z_obs, x_obs_2, y_obs_2, z_obs_2]; 

% Assuming spherical obstacle, with the following radius
radius_obs = 10;

% Minimum accepted distance from obstacle in meters
min_dist = radius_obs + 5; 