%% Path Selector
%% Description
% *************************************************************************
% This file does post-optimization path selection by selecting a path that
% fulfils requirements in terms of desired path properties on objective
% function values.
%
% This file searches for path candidates in the final population.
% *************************************************************************
% Programmed by: Tobias Lars Hansen, 2019.
% *************************************************************************
%%
clear PP;
PP = P;
%% Scaling factors
beta_o2 = 0.8;
beta_o4 = 1;

%% Computations
avg_obj_1 = sum(PP(:,V*(K+3)+1))/N;
avg_obj_2 = sum(PP(:,V*(K+3)+2))/N;
avg_obj_3 = sum(PP(:,V*(K+3)+3))/N;
avg_obj_4 = sum(PP(:,V*(K+3)+4))/N;

min_safety_margin = beta_o2 * avg_obj_2;
min_terrain_factor = beta_o4 * avg_obj_4;

a_shortest_path = min(PP(:,V*(K+3)+1));
a_longest_path = max(PP(:,V*(K+3)+1));

ideal_length = (avg_obj_1 + a_shortest_path)/2;

flag = 0;
counter_discarded = 0;
while flag ~= 1
    [~, a_index] = min(abs (ideal_length - PP(:,V*(K+3)+1)));
    safety_factor = PP(a_index,V*(K+3)+2);
    terrain_factor = PP(a_index,V*(K+3)+4);
    
    if safety_factor > min_safety_margin ||...
            terrain_factor > min_terrain_factor
        
        PP(a_index, V*(K+3) + 1) = a_longest_path;
        counter_discarded = counter_discarded + 1;
        
    else
        flag = 1;
    end 
end