close all;
%% Description
% *************************************************************************
% Post-optimization visualisation of performance.
% *************************************************************************
% Programmed by: Tobias Lars Hansen, 2019.
% *************************************************************************
%% Plotting objective function values
nbPath = a_index;

x_points = P(:, V*(K+3) + 1);
y_points = P(:, V*(K+3) + 2);
z_points = P(:, V*(K+3) + 3);
zz_points = P(:, V*(K+3) + 4);

x_ccc = P(nbPath, V*(K+3) + 1);
y_ccc = P(nbPath, V*(K+3) + 2);
z_ccc = P(nbPath, V*(K+3) + 3);
zz_ccc = P(nbPath, V*(K+3) + 4);

%% 3D plots: Objective function values
%% O_1, O_2, O_3
figure()
scatter3(x_points,y_points, z_points); hold on;
xlabel('Objective function 1');
ylabel('Objective function 2');
zlabel('Objective function 3');
schosen = scatter3(x_ccc,y_ccc,z_ccc, 'r', 'filled');
LL = legend(schosen,{'Chosen solution'});

%% O_1, O_2, O_4
figure()
scatter3(x_points,y_points, zz_points); hold on;
schosen = scatter3(x_ccc,y_ccc,zz_ccc, 'r', 'filled');
LL = legend(schosen,{'Chosen solution'});
xlabel('Objective function 1');
ylabel('Objective function 2');
zlabel('Objective function 4');

%% O_1, O_3, O_4
figure()
scatter3(x_points,z_points, zz_points); hold on;
schosen = scatter3(x_ccc,z_ccc,zz_ccc, 'r', 'filled');
LL = legend(schosen,{'Chosen solution'});
xlabel('Objective function 1');
ylabel('Objective function 3');
zlabel('Objective function 4');

%% O_2, O_3, O_4
figure()
scatter3(y_points,z_points, zz_points); hold on;
schosen = scatter3(y_ccc,z_ccc,zz_ccc, 'r', 'filled');
LL = legend(schosen,{'Chosen solution'});
xlabel('Objective function 2');
ylabel('Objective function 3');
zlabel('Objective function 4');

%% 2D plots: Objective function values
%% O_1, O_2
figure()
scatter(x_points,y_points); hold on;
sc = scatter(x_ccc,y_ccc, 'r', 'filled');
LLL = legend(sc,{'Chosen solution'});
xlabel('Objective function 1');
ylabel('Objective function 2');
grid on;

%% O_1, O_3
figure()
scatter(x_points,z_points); hold on;
sc = scatter(x_ccc,z_ccc, 'r', 'filled');
LLL = legend(sc,{'Chosen solution'});
xlabel('Objective function 1');
ylabel('Objective function 3');
grid on;

%% O_1, O_4
figure()
scatter(x_points,zz_points); hold on;
sc = scatter(x_ccc,zz_ccc, 'r', 'filled');
LLL = legend(sc,{'Chosen solution'});
xlabel('Objective function 1');
ylabel('Objective function 4');
grid on;

%% O_2, O_3
figure()
scatter(y_points,z_points); hold on;
sc = scatter(y_ccc,z_ccc, 'r', 'filled');
LLL = legend(sc,{'Chosen solution'});
xlabel('Objective function 2');
ylabel('Objective function 3');
grid on;
%% O_2, O_4
figure()
scatter(y_points,zz_points); hold on;
sc = scatter(y_ccc,zz_ccc, 'r', 'filled');
LLL = legend(sc,{'Chosen solution'});
xlabel('Objective function 2');
ylabel('Objective function 4');
grid on;

%% O_3, O_4
figure()
scatter(z_points,zz_points); hold on;
sc = scatter(z_ccc,zz_ccc, 'r', 'filled');
LLL = legend(sc,{'Chosen solution'});
xlabel('Objective function 3');
ylabel('Objective function 4');
grid on;

%% Plotting the selected path
% selected by path selector
figure()
nbWps = K + 3;
xc = 1;
yc = 2;
zc = 3;
Path = P(nbPath, :);

% Waypoints
s0 = scatter3(Path(xc), Path(yc), Path(zc), 'b', 'filled'); hold on;
xc = 1 + V;
yc = 2 + V;
zc = 3 + V;
for wp = 2: nbWps-1
    s = scatter3(Path(xc), Path(yc), Path(zc), 'k', 'filled');
    hold on; 
    s.LineWidth = 0.6;
    xc = xc + V;
    yc = yc + V;
    zc = zc + V;
end
st = scatter3(Path(xc), Path(yc), Path(zc), 'g', 'filled'); hold on;

% Line segments
xc = 1;
yc = 2;
zc = 3;
for n = 1:nbWps
    X(n) = Path(xc);
    Y(n) = Path(yc);
    Z(n) = Path(zc);
    xc = xc + V;
    yc = yc + V;
    zc = zc + V;
end
p = plot3(X,Y,Z, 'r'); hold on;
p.LineWidth = 2;
set( gca, 'Zdir', 'reverse');
set( gca, 'Ydir', 'reverse');
grid on; 

% Obstacles
[XX, YY, ZZ] = sphere;
surf(radius_obs*XX+obs_vec(1),...
    radius_obs*YY+obs_vec(2),...
    radius_obs*ZZ+obs_vec(3) );

surf(radius_obs*XX+obs_vec(4),...
    radius_obs*YY+obs_vec(5),...
    radius_obs*ZZ+obs_vec(6) );

% Seabed
% xd=linspace(0,150);
% yd=linspace(0,150);
% [xxx,yyy]=meshgrid(xd,yd);
% zzz = 0.007*xxx.^2  + 0.007*yyy.^2 + 0.05*xxx + 0.05*yyy + 40;
% CC = del2(zzz);
% sbed = mesh(xxx,yyy,zzz);

% View
% 3D settings: None

% NE settings
%view([0 0 1]);
%camroll(-90);
%view(2);

% ED settings
%view([-1 0 0]);

% Layout
xlim([0, 100]);
ylim([0, 100]); 
zlim([0,200]);

L = legend([s0 st], {'Initial position', 'Terminal position'});
xlabel('North [m]');
ylabel('East [m]');
zlabel('Down [m]');