function f = selection (intermediate_pop, N, V, K, M)
%% Description
% *************************************************************************
% This function performs selection of the next parent population based on
% rank and crowding distance. 
%
% This function calls the function BTS if paths cannot be distinguished
% from each other in terms of rank and crowding distance.
% 
% Input: Intermediate population - size 2*N
% Output: Parent population for next iteration - size N
% *************************************************************************
% Programmed by: Tobias Lars Hansen, 2019.
% *************************************************************************
%%
rank_f = intermediate_pop(100,V*(K+2)+M+1);

for i = 1:2*N
    rank_i = intermediate_pop(i,V*(K+2)+M+1);
    if rank_i == rank_f
        final_front(i,:) = intermediate_pop(i,:);
    end
end

nb_prev_sol = 0;
for i = 1:length(final_front)
    row = sum(final_front(i,:));
    if row == 0
        nb_prev_sol = nb_prev_sol + 1;
    end
end

pop_next(1:nb_prev_sol,:) = intermediate_pop(1:nb_prev_sol,:);
pool_size = N-length(pop_next);

if pool_size > 0
    contenders = intermediate_pop(nb_prev_sol+1:nb_prev_sol+2*pool_size,:);
    winners = BTS(contenders, pool_size, V, K, M);
    winners = sortrows(winners, V*(K+2)+M+1);
    pop_next(nb_prev_sol+1:nb_prev_sol+pool_size,:) = winners;
end

f = pop_next;
end %fcn